/*  FUNNEL LIBRARY
**  
**  Console -- a library that interfaces with text consoles
**  
**  11/06/2K, Matthias Zenger
*/

type JavaObject = any

newtype ConsoleModule = {
    def in: JavaObject
    def out: JavaObject
    def instream: JavaObject
    def print(x: any): ()
    def println(x: any): ()
    def readln: string
    def readInt: int
    def readFloat: float
}

/** a synchronous implementation of console i/o; i.e. the
 *  input functions are blocking
 */
let Console: ConsoleModule = {
    val in = primitive (3, "java.lang.System", "in", "Ljava/io/InputStream;") ()
    val out = primitive (3, "java.lang.System", "out", "Ljava/io/PrintStream;") ()

    val instream =
      primitive (0, "java.io.BufferedReader", "<init>", "(Ljava/io/Reader;)V")
	(primitive (0, "java.io.InputStreamReader", "<init>", "(Ljava/io/InputStream;)V")
	  (in))

    def print (x: any) =
      primitive (2, "java.io.PrintStream", "print", "(Ljava/lang/Object;)V") (out, x)
    def println (x: any) =
      primitive (2, "java.io.PrintStream", "println", "(Ljava/lang/Object;)V") (out, x)

    def readln =
      primitive (2, "java.io.BufferedReader", "readLine", "()Ljava/lang/String;") (instream)
    def readInt = String.toInt readln
    def readFloat = String.toFloat readln
}
